@@ -16,12 +16,12 @@ use Business::CreditCard qw(cardtype);
 use Data::Dumper;
 use IO::String;
 use Carp qw(croak);
-use Log::Scrubber qw(disable $SCRUBBER scrubber :Carp);
+use Log::Scrubber qw(disable $SCRUBBER scrubber :Carp scrubber_add_scrubber);
 
 @ISA     = qw(Business::OnlinePayment::HTTPS);
 $me      = 'Business::OnlinePayment::Litle';
 $DEBUG   = 0;
-$VERSION = '0.932';
+$VERSION = '0.933';
 
 =head1 NAME
 
@@ -88,10 +88,62 @@ Returns the response error description text.
 
 Returns the complete request that was sent to the server.  The request has been stripped of card_num, cvv2, and password.  So it should be safe to log.
 
+=cut
+
+sub server_request {
+    my ( $self, $val, $tf ) = @_;
+    if ($val) {
+        $self->{server_request} = scrubber $val;
+        $self->server_request_dangerous($val,1) unless $tf;
+    }
+    return $self->{server_request};
+}
+
+=head2 server_request_dangerous
+
+Returns the complete request that was sent to the server.  This could contain data that is NOT SAFE to log.  It should only be used in a test environment, or in a PCI compliant manner.
+
+=cut
+
+sub server_request_dangerous {
+    my ( $self, $val, $tf ) = @_;
+    if ($val) {
+        $self->{server_request_dangerous} = $val;
+        $self->server_request($val,1) unless $tf;
+    }
+    return $self->{server_request_dangerous};
+}
+
 =head2 server_response
 
 Returns the complete response from the server.  The response has been stripped of card_num, cvv2, and password.  So it should be safe to log.
 
+=cut
+
+sub server_response {
+    my ( $self, $val, $tf ) = @_;
+    if ($val) {
+        $self->{server_response} = scrubber $val;
+        $self->server_response_dangerous($val,1) unless $tf;
+    }
+    return $self->{server_response};
+}
+
+=head2 server_response_dangerous
+
+Returns the complete response from the server.  This could contain data that is NOT SAFE to log.  It should only be used in a test environment, or in a PCI compliant manner.
+
+=cut
+
+sub server_response_dangerous {
+    my ( $self, $val, $tf ) = @_;
+    if ($val) {
+        $self->{server_response_dangerous} = $val;
+        $self->server_response($val,1) unless $tf;
+    }
+    return $self->{server_response_dangerous};
+}
+
 =head1 Handling of content(%content) data:
 
 =head2 action
@@ -190,7 +242,7 @@ sub set_defaults {
         qw( order_number md5 avs_code cvv2_response
           cavv_response api_version xmlns failure_status batch_api_version chargeback_api_version
           is_prepaid prepaid_balance get_affluence chargeback_server chargeback_port chargeback_path
-          verify_SSL phoenixTxnId server_request server_response
+          verify_SSL phoenixTxnId
           )
     );
 
@@ -375,7 +427,9 @@ sub map_fields {
 
 =head2 format_misc_field
 
+A new method not directly supported by BOP.
 Used internally to guarentee that XML data will conform to the Litle spec.
+  field  - The hash key we are checking against
   maxLen - The maximum length allowed (extra bytes will be truncated)
   minLen - The minimum length allowed
   errorOnLength - boolean
@@ -417,6 +471,7 @@ sub format_misc_field {
 
 =head2 format_amount_field
 
+A new method not directly supported by BOP.
 Used internally to change amounts from the BOP "5.00" format to the format expected by Litle "500"
 
 $tx->format_amount_field( \%content, 'amount' );
@@ -433,6 +488,7 @@ sub format_amount_field {
 
 =head2 format_phone_field
 
+A new method not directly supported by BOP.
 Used internally to strip invalid characters from phone numbers. IE "1 (800).TRY-THIS" becomes "18008788447"
 
 $tx->format_phone_field( \%content, 'company_phone' );
@@ -456,6 +512,12 @@ sub format_phone_field {
     }
 }
 
+=head2 map_request
+
+Converts the BOP data to something that Litle can use.
+
+=cut
+
 sub map_request {
     my ( $self, $content ) = @_;
 
@@ -482,9 +544,14 @@ sub map_request {
     if ( ! defined $content->{'description'} ) { $content->{'description'} = ''; } # shema req
     $content->{'description'} =~ s/[^\w\s\*\,\-\'\#\&\.]//g;
 
+    # Litle pre 0.934 used token, however BOP likes card_token
+    $content->{'card_token'} = $content->{'token'} if ! defined $content->{'card_token'} && defined $content->{'card_token'};
+
     # only numbers are allowed in company_phone
     $self->format_phone_field($content, 'company_phone');
 
+    $content->{'invoice_number_length_15'} ||= $content->{'invoice_number'}; # orderId = 25, invoiceReferenceNumber = 15
+
     #  put in a list of constraints
     my @validate = (
       # field,     maxLen, minLen, errorOnLength, isRequired
@@ -516,7 +583,8 @@ sub map_request {
       [ 'discount',     8,      0,             1, 0 ],
       [ 'shipping',     8,      0,             1, 0 ],
       [ 'duty',         8,      0,             1, 0 ],
-      ['invoice_number',15,     0,             0, 0 ], # TODO orderID = 25, invoiceReferenceNumber = 15
+      ['invoice_number',25,     0,             0, 0 ],
+      ['invoice_number_length_15',15,0,        0, 0 ],
       [ 'orderdate',   10,      0,             0, 0 ], # YYYY-MM-DD
 
       [ 'recycle_by',   8,      0,             0, 0 ],
@@ -528,7 +596,7 @@ sub map_request {
       [ 'card_number', 25,     13,             1, 0 ],
       [ 'expiration',   4,      4,             1, 0 ], # MMYY
       [ 'cvv2',         4,      3,             1, 0 ],
-      # 'token' does not have a documented limit
+      # 'card_token' does not have a documented limit
 
       [ 'customer_id', 25,      0,             0, 0 ],
     );
@@ -537,7 +605,7 @@ sub map_request {
       #warn "$trunc->[0] => ".($content->{ $trunc->[0] }||'')."\n" if $DEBUG;
     }
 
-    tie my %billToAddress, 'Tie::IxHash', $self->revmap_fields(
+    tie my %billToAddress, 'Tie::IxHash', $self->_revmap_fields(
         content      => $content,
         name         => 'name',
         email        => 'email',
@@ -550,7 +618,7 @@ sub map_request {
         phone => 'phone',
     );
 
-    tie my %shipToAddress, 'Tie::IxHash', $self->revmap_fields(
+    tie my %shipToAddress, 'Tie::IxHash', $self->_revmap_fields(
         content      => $content,
         name         => 'ship_name',
         email        => 'ship_email',
@@ -564,13 +632,13 @@ sub map_request {
     );
 
     tie my %customerinfo, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content      => $content,
         customerType => 'customerType',
       );
 
     tie my %custombilling, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content      => $content,
         phone      => 'company_phone',
         descriptor => 'description',
@@ -601,7 +669,7 @@ sub map_request {
           foreach my $trunc ( @validate ) { $self->format_misc_field(\%prod,$trunc); }
 
           tie my %lineitem, 'Tie::IxHash',
-            $self->revmap_fields(
+            $self->_revmap_fields(
               content              => \%prod,
               itemSequenceNumber   => 'itemSequenceNumber',
               itemDescription      => 'description',
@@ -621,19 +689,19 @@ sub map_request {
 
     #
     #
-    tie my %enhanceddata, 'Tie::IxHash', $self->revmap_fields(
+    tie my %enhanceddata, 'Tie::IxHash', $self->_revmap_fields(
         content                => $content,
         customerReference      => 'po_number',
         salesTax               => 'salestax',
         discountAmount         => 'discount',
         shippingAmount         => 'shipping',
         dutyAmount             => 'duty',
-        invoiceReferenceNumber => 'invoice_number',    ##
+        invoiceReferenceNumber => 'invoice_number_length_15',
         orderDate              => 'orderdate',
         lineItemData           => \@products,
     );
 
-    tie my %card, 'Tie::IxHash', $self->revmap_fields(
+    tie my %card, 'Tie::IxHash', $self->_revmap_fields(
         content            => $content,
         type               => 'card_type',
         number             => 'card_number',
@@ -641,20 +709,20 @@ sub map_request {
         cardValidationNum  => 'cvv2',
     );
 
-    tie my %token, 'Tie::IxHash', $self->revmap_fields(
+    tie my %token, 'Tie::IxHash', $self->_revmap_fields(
         content            => $content,
-        litleToken         => 'token',
+        litleToken         => 'card_token',
         expDate            => 'expiration',
         cardValidationNum  => 'cvv2',
     );
 
-    tie my %processing, 'Tie::IxHash', $self->revmap_fields(
+    tie my %processing, 'Tie::IxHash', $self->_revmap_fields(
         content               => $content,
         bypassVelocityCheck   => 'velocity_check',
     );
 
     tie my %cardholderauth, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content                     => $content,
         authenticationValue         => '3ds',
         authenticationTransactionId => 'visaverified',
@@ -663,13 +731,14 @@ sub map_request {
       );
 
     tie my %merchantdata, 'Tie::IxHash',
-      $self->revmap_fields(
-        content      => $content,
-        affiliate    => 'affiliate',
+      $self->_revmap_fields(
+        content            => $content,
+        affiliate          => 'affiliate',
+        merchantGroupingId => 'merchant_grouping_id',
       );
 
     tie my %recyclingrequest, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content      => $content,
         recycleBy    => 'recycle_by',
         recycleId    => 'recycle_id',
@@ -678,14 +747,15 @@ sub map_request {
     my %req;
 
     if ( $action eq 'sale' ) {
-        tie %req, 'Tie::IxHash', $self->revmap_fields(
+        croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
+        tie %req, 'Tie::IxHash', $self->_revmap_fields(
             content       => $content,
             orderId       => 'invoice_number',
             amount        => 'amount',
             orderSource   => 'orderSource',
             billToAddress => \%billToAddress,
-            card          => \%card,
-            token         => $content->{'token'} ? \%token : {},
+            card          => $content->{'card_number'} ? \%card : {},
+            token         => $content->{'card_token'} ? \%token : {},
 
             #cardholderAuthentication    =>  \%cardholderauth,
             customBilling => \%custombilling,
@@ -697,14 +767,15 @@ sub map_request {
         );
     }
     elsif ( $action eq 'authorization' ) {
-        tie %req, 'Tie::IxHash', $self->revmap_fields(
+        croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
+        tie %req, 'Tie::IxHash', $self->_revmap_fields(
             content       => $content,
             orderId       => 'invoice_number',
             amount        => 'amount',
             orderSource   => 'orderSource',
             billToAddress => \%billToAddress,
-            card          => \%card,
-            token         => $content->{'token'} ? \%token : {},
+            card          => $content->{'card_number'} ? \%card : {},
+            token         => $content->{'card_token'} ? \%token : {},
 
             #cardholderAuthentication    =>  \%cardholderauth,
             processingInstructions  =>  \%processing,
@@ -717,7 +788,7 @@ sub map_request {
     elsif ( $action eq 'capture' ) {
         push @required_fields, qw( order_number amount );
         tie %req, 'Tie::IxHash',
-          $self->revmap_fields(
+          $self->_revmap_fields(
             content      => $content,
             litleTxnId   => 'order_number',
             amount       => 'amount',
@@ -730,7 +801,7 @@ sub map_request {
        # IF there is a litleTxnId, it's a normal linked credit
        if( $content->{'order_number'} ){
           push @required_fields, qw( order_number amount );
-          tie %req, 'Tie::IxHash', $self->revmap_fields(
+          tie %req, 'Tie::IxHash', $self->_revmap_fields(
               content       => $content,
               litleTxnId    => 'order_number',
               amount        => 'amount',
@@ -740,15 +811,16 @@ sub map_request {
         }
        # ELSE it's an unlinked, which requires different data
        else {
+          croak 'missing card_token or card_number' if length($content->{'card_number'} || $content->{'card_token'} || '') == 0;
           push @required_fields, qw( invoice_number amount );
-          tie %req, 'Tie::IxHash', $self->revmap_fields(
+          tie %req, 'Tie::IxHash', $self->_revmap_fields(
               content       => $content,
               orderId       => 'invoice_number',
               amount        => 'amount',
               orderSource   => 'orderSource',
               billToAddress => \%billToAddress,
-              card          => \%card,
-              token         => $content->{'token'} ? \%token : {},
+              card          => $content->{'card_number'} ? \%card : {},
+              token         => $content->{'card_token'} ? \%token : {},
               customBilling => \%custombilling,
               processingInstructions => \%processing,
           );
@@ -757,7 +829,7 @@ sub map_request {
     elsif ( $action eq 'void' ) {
         push @required_fields, qw( order_number );
         tie %req, 'Tie::IxHash',
-          $self->revmap_fields(
+          $self->_revmap_fields(
             content                 => $content,
             litleTxnId              => 'order_number',
             processingInstructions  =>  \%processing,
@@ -766,7 +838,7 @@ sub map_request {
     elsif ( $action eq 'authReversal' ) {
         push @required_fields, qw( order_number amount );
         tie %req, 'Tie::IxHash',
-          $self->revmap_fields(
+          $self->_revmap_fields(
             content    => $content,
             litleTxnId => 'order_number',
             amount     => 'amount',
@@ -775,7 +847,7 @@ sub map_request {
     elsif ( $action eq 'accountUpdate' ) {
         push @required_fields, qw( card_number expiration );
         tie %req, 'Tie::IxHash',
-          $self->revmap_fields(
+          $self->_revmap_fields(
             content => $content,
             orderId => 'customer_id',
             card    => \%card,
@@ -810,7 +882,7 @@ sub submit {
 
     ## set the authentication data
     tie my %authentication, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content  => \%content,
         user     => 'login',
         password => 'password',
@@ -850,15 +922,15 @@ sub submit {
     $writer->end();
     ## END XML Generation
 
-    $self->server_request( scrubber $post_data );
+    $self->server_request( $post_data );
     warn $self->server_request if $DEBUG;
 
     my ( $page, $status_code, %headers ) = $self->https_post( { 'Content-Type' => 'text/xml;charset:utf-8' } , $post_data);
 
-    $self->server_response( scrubber $page );
+    $self->server_response( $page );
     warn Dumper $self->server_response, $status_code, \%headers if $DEBUG;
 
-    my $response = $self->parse_xml_response( $page, $status_code );
+    my $response = $self->_parse_xml_response( $page, $status_code );
     if ( exists( $response->{'response'} ) && $response->{'response'} == 1 ) {
         ## parse error type error
         warn Dumper $response, $self->server_request;
@@ -929,12 +1001,13 @@ sub submit {
 
 =head2 chargeback_retrieve_support_doc
 
+A new method not directly supported by BOP.
 Retrieve a currently uploaded file
 
  $tx->content(
   login       => 'testdrive',
   password    => '123qwe',
-  mercahntid  => '123456',
+  merchantid  => '123456',
   case_id     => '001',
   filename    => 'mydoc.pdf',
  );
@@ -945,29 +1018,31 @@ Retrieve a currently uploaded file
 
 sub chargeback_retrieve_support_doc {
     my ( $self ) = @_;
-    $self->litle_support_doc('RETRIEVE');
+    $self->_litle_support_doc('RETRIEVE');
     if ($self->is_success) { $self->{'fileContent'} = $self->{'_response'}; } else { $self->{'fileContent'} = undef; }
 }
 
 =head2 chargeback_delete_support_doc
 
+A new method not directly supported by BOP.
 Delete a currently uploaded file.  Follows the same format as chargeback_retrieve_support_doc
 
 =cut
 
 sub chargeback_delete_support_doc {
     my ( $self ) = @_;
-    $self->litle_support_doc('DELETE' );
+    $self->_litle_support_doc('DELETE' );
 }
 
 =head2 chargeback_upload_support_doc
 
+A new method not directly supported by BOP.
 Upload a new file
 
  $tx->content(
   login       => 'testdrive',
   password    => '123qwe',
-  mercahntid  => '123456',
+  merchantid  => '123456',
   case_id     => '001',
   filename    => 'mydoc.pdf',
   filecontent => $binaryPdfData,
@@ -979,21 +1054,22 @@ Upload a new file
 
 sub chargeback_upload_support_doc {
     my ( $self ) = @_;
-    $self->litle_support_doc('UPLOAD' );
+    $self->_litle_support_doc('UPLOAD' );
 }
 
 =head2 chargeback_replace_support_doc
 
+A new method not directly supported by BOP.
 Replace a previously uploaded file.  Follows the same format as chargeback_upload_support_doc
 
 =cut
 
 sub chargeback_replace_support_doc {
     my ( $self ) = @_;
-    $self->litle_support_doc('REPLACE' );
+    $self->_litle_support_doc('REPLACE' );
 }
 
-sub litle_support_doc {
+sub _litle_support_doc {
     my ( $self, $action ) = @_;
 
     $self->is_success(0);
@@ -1021,7 +1097,7 @@ sub litle_support_doc {
       if ( defined $content{'filecontent'} ) {
           if ( length($content{'filecontent'}) > 2097152 ) { # file limit of 2M
               my $msg = 'Filesize Exceeds Limit Of 2MB';
-              $self->result_code( 012 );
+              $self->result_code( 012 ); ## no critic
               $self->error_message( $msg );
               croak $msg;
           }
@@ -1054,14 +1130,14 @@ sub litle_support_doc {
         content => $content{'filecontent'},
     } );
 
-    $self->server_request( scrubber $content{'mimetype'} );
-    $self->server_response( scrubber $response->{'content'} );
+    $self->server_request( $content{'mimetype'} );
+    $self->server_response( $response->{'content'} );
 
     if ( $action eq 'RETRIEVE' && $response->{'status'} =~ /^200/ && substr($response->{'content'},0,500) !~ /<Merchant/x) {
         # the RETRIEVE action returns the actual page as the file, rather then returning XML
         $self->is_success(1);
     } else {
-        my $xml_response = $self->parse_xml_response( $response->{'content'}, $response->{'status'} );
+        my $xml_response = $self->_parse_xml_response( $response->{'content'}, $response->{'status'} );
 
         if (defined $xml_response && defined $xml_response->{'ChargebackCase'}{'Document'}{'ResponseCode'}) {
             $self->is_success( $xml_response->{'ChargebackCase'}{'Document'}{'ResponseCode'} eq '000' ? 1 : 0 );
@@ -1073,14 +1149,15 @@ sub litle_support_doc {
     }
 }
 
-=head2 list_support_docs
+=head2 chargeback_list_support_docs
 
+A new method not directly supported by BOP.
 Return a hashref that contains a list of files that already exist on the server.
 
  $tx->content(
   login       => 'testdrive',
   password    => '123qwe',
-  mercahntid  => '123456',
+  merchantid  => '123456',
   case_id     => '001',
  );
  my $ret = $tx->chargeback_list_support_docs();
@@ -1118,10 +1195,10 @@ sub chargeback_list_support_docs {
         headers => { Authorization => 'Basic ' . MIME::Base64::encode("$content{'login'}:$content{'password'}",'') },
     } );
 
-    $self->server_request( scrubber $url );
-    $self->server_response( scrubber $response->{'content'} );
+    $self->server_request( $url );
+    $self->server_response( $response->{'content'} );
 
-    my $xml_response = $self->parse_xml_response( $response->{'content'}, $response->{'status'} );
+    my $xml_response = $self->_parse_xml_response( $response->{'content'}, $response->{'status'} );
 
     if (defined $xml_response && $xml_response->{'ChargebackCase'}{'ResponseCode'}) {
         $self->result_code( $xml_response->{'ChargebackCase'}{'ResponseCode'} );
@@ -1142,7 +1219,7 @@ sub chargeback_list_support_docs {
     return {};
 }
 
-sub parse_xml_response {
+sub _parse_xml_response {
     my ( $self, $page, $status_code ) = @_;
     my $response = {};
     if ( $status_code =~ /^200/ ) {
@@ -1158,7 +1235,7 @@ sub parse_xml_response {
     return $response;
 }
 
-sub parse_batch_response {
+sub _parse_batch_response {
     my ( $self, $args ) = @_;
     my @results;
     my $resp = $self->{'batch_response'};
@@ -1169,15 +1246,25 @@ sub parse_batch_response {
       grep { $_ =~ m/Response$/ }
       keys %{$resp};    ## get a list of result types in this batch
     return {
-        'account_update' => $self->get_update_response,
+        'account_update' => $self->_get_update_response,
         ## do the other response types now
     };
 }
 
 =head2 add_item
 
-A new method, not supported under BOP yet, but interface to adding multiple entries, so we can write and interface with batches
+A new method not directly supported by BOP.
+Interface to adding multiple entries, so we can write and interface with batches
 
+ my %content = (
+   action          =>  'Account Update',
+   card_number     =>  4111111111111111,
+   expiration      =>  1216,
+   customer_id     =>  $card->{'uid'},
+   invoice_number  =>  123,
+   type            =>  'VI',
+   login           =>  $merchant->{'login'},
+ );
  $tx->add_item( \%content );
 
 =cut
@@ -1188,16 +1275,43 @@ sub add_item {
     push @{ $self->{'batch_entries'} }, shift;
 }
 
+=head2 create_batch
+
+A new method not directly supported by BOP.
+Send the current batch to Litle.
+
+ $tx->add_item( $item );
+ $tx->add_item( $item );
+ $tx->add_item( $item );
+
+ my $opts = {
+  login       => 'testdrive',
+  password    => '123qwe',
+  merchantid  => '123456',
+  batch_id    => '001',
+  method      => 'https', # sftp or https
+  ftp_username=> 'fred',
+  ftp_password=> 'pancakes',
+ };
+
+ $tx->content();
+
+ $tx->create_batch( %$opts );
+
+=cut
+
 sub create_batch {
     my ( $self, %opts ) = @_;
 
     $self->is_success(0);
+    $self->server_request('');
+    $self->server_response('');
 
     local $SCRUBBER=1;
-    $self->_litle_scrubber_init;
+    $self->_litle_scrubber_init(\%opts);
 
-    if ( scalar( @{ $self->{'batch_entries'} } ) < 1 ) {
-        $self->error('Cannot create an empty batch');
+    if ( ! defined $self->{'batch_entries'} || scalar( @{ $self->{'batch_entries'} } ) < 1 ) {
+        $self->error_message('Cannot create an empty batch');
         return;
     }
 
@@ -1211,7 +1325,7 @@ sub create_batch {
     );
     ## set the authentication data
     tie my %authentication, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content  => \%opts,
         user     => 'login',
         password => 'password',
@@ -1238,6 +1352,7 @@ sub create_batch {
     );
     foreach my $entry ( @{ $self->{'batch_entries'} } ) {
         $self->content( %{$entry} );
+        $self->_litle_scrubber_add_card($entry->{'card_number'});
         my %content = $self->content;
         my $req     = $self->map_request( \%content );
         $writer->startTag(
@@ -1257,6 +1372,9 @@ sub create_batch {
     $writer->end();
     ## END XML Generation
 
+    $self->server_request( $post_data );
+    warn $self->server_request if $DEBUG;
+
     #----- Send it
     if ( $opts{'method'} && $opts{'method'} eq 'sftp' ) {    #FTP
         require Net::SFTP::Foreign;
@@ -1265,7 +1383,7 @@ sub create_batch {
             user     => $opts{'ftp_username'},
             password => $opts{'ftp_password'},
         );
-        $sftp->error and die "SSH connection failed: " . $sftp->error;
+        $sftp->error and die "SSH connection failed: " . $sftp->error . "\n";
 
         $sftp->setcwd("inbound")
           or die "Cannot change working directory ", $sftp->error;
@@ -1278,15 +1396,14 @@ sub create_batch {
             "$filename.asc" ) #once complete, you rename it, for pickup
           or die "Cannot RENAME file", $sftp->message;
         $self->is_success(1);
+        $self->server_response( $sftp->message );
     }
     elsif ( $opts{'method'} && $opts{'method'} eq 'https' ) {    #https post
         $self->port('15000');
         $self->path('/');
         my ( $page, $status_code, %headers ) =
           $self->https_post($post_data);
-        $self->server_request( scrubber $post_data );
-        $self->server_response( scrubber $page );
-        warn $self->server_request if $DEBUG;
+        $self->server_response( $page );
 
         warn Dumper [ $page, $status_code, \%headers ] if $DEBUG;
 
@@ -1339,6 +1456,12 @@ sub create_batch {
 
 }
 
+=head2 send_rfr
+
+A new method not directly supported by BOP.
+
+=cut
+
 sub send_rfr {
     my ( $self, $args ) = @_;
     my $post_data;
@@ -1355,7 +1478,7 @@ sub send_rfr {
     );
     ## set the authentication data
     tie my %authentication, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content  => $args,
         user     => 'login',
         password => 'password',
@@ -1391,8 +1514,8 @@ sub send_rfr {
     $self->path('/');
     my ( $page, $status_code, %headers ) = $self->https_post($post_data);
 
-    $self->server_request( scrubber $post_data );
-    $self->server_response( scrubber $page );
+    $self->server_request( $post_data );
+    $self->server_response( $page );
     warn $self->server_request if $DEBUG;
 
     warn Dumper [ $page, $status_code, \%headers ] if $DEBUG;
@@ -1428,10 +1551,30 @@ sub send_rfr {
         my $resp = $response->{'batchResponse'};
         $self->is_success( $resp->{'response'} eq '000' ? 1 : 0 );
         $self->{'batch_response'} = $resp;
-        $self->parse_batch_response;
+        $self->_parse_batch_response;
     }
 }
 
+=head2 retrieve_batch
+
+A new method not directly supported by BOP.
+Get a batch from Litle.
+
+ my $opts = {
+  login       => 'testdrive',
+  password    => '123qwe',
+  batch_id    => '001',
+  batch_return=> '',
+  ftp_username=> 'fred',
+  ftp_password=> 'pancakes',
+ };
+
+ $tx->content();
+
+ $tx->retrieve_batch( %$opts );
+
+=cut
+
 sub retrieve_batch {
     my ( $self, %opts ) = @_;
 
@@ -1478,8 +1621,8 @@ sub retrieve_batch {
         $self->error_message( $response->{'batchResponse'}->{'message'} );
     }
 
-    $self->server_request( scrubber $post_data );
-    $self->server_response( scrubber $response );
+    $self->server_request( $post_data );
+    $self->server_response( $response );
 
     $self->{_response} = $response;
     my $resp = $response->{'batchResponse'};
@@ -1488,11 +1631,11 @@ sub retrieve_batch {
     $self->is_success( $response->{'response'} eq '0' ? 1 : 0 );
     if ( $self->is_success() ) {
         $self->{'batch_response'} = $resp;
-        return $self->parse_batch_response;
+        return $self->_parse_batch_response;
     }
 }
 
-sub get_update_response {
+sub _get_update_response {
     my $self = shift;
     require Business::OnlinePayment::Litle::UpdaterResponse;
     my @response;
@@ -1505,7 +1648,7 @@ sub get_update_response {
     return \@response;
 }
 
-sub revmap_fields {
+sub _revmap_fields {
     my $self = shift;
     tie my (%map), 'Tie::IxHash', @_;
     my %content;
@@ -1513,7 +1656,7 @@ sub revmap_fields {
         %content = %{ delete( $map{'content'} ) };
     }
     else {
-        warn "WARNING: This content has no been pre-processed with map_fields";
+        warn "WARNING: This content has not been pre-processed with map_fields";
         %content = $self->content();
     }
 
@@ -1559,24 +1702,43 @@ sub _xmlwrite {
     }
     else {
         $writer->startTag($item);
-        utf8::decode($value); # prevent double byte corruption in the xml output
         $writer->characters($value);
         $writer->endTag($item);
     }
 }
 
+sub _litle_scrubber_add_card {
+    my ( $self, $cc ) = @_;
+    return if ! $cc;
+    my $del = substr($cc,0,6).('X'x(length($cc)-10)).substr($cc,-4,4); # show first 6 and last 4
+    scrubber_add_scrubber({$cc=>$del});
+}
+
 sub _litle_scrubber_init {
-    my ( $self ) = @_;
+    my ( $self, $opts ) = @_;
     my %content = $self->content();
+    if ($opts) { %content = %$opts; }
     scrubber_init({
         quotemeta($content{'password'}||'')=>'DELETED',
         quotemeta($content{'ftp_password'}||'')=>'DELETED',
-        quotemeta($content{'card_number'}||'')=>'DELETED',
         ($content{'cvv2'} ? '(?<=[^\d])'.quotemeta($content{'cvv2'}).'(?=[^\d])' : '')=>'DELETED',
         });
+    $self->_litle_scrubber_add_card($content{'card_number'});
 }
 
-#------------------------------------ Chargebacks
+=head2 chargeback_activity_request
+
+Return a arrayref that contains a list of Business::OnlinePayment::Litle::ChargebackActivityResponse objects
+
+ $tx->content(
+  login         => 'testdrive',
+  password      => '123qwe',
+  activity_date => '2012-04-30',
+ );
+
+ my $ret = $tx->chargeback_activity_request();
+
+=cut
 
 sub chargeback_activity_request {
     my ( $self ) = @_;
@@ -1617,7 +1779,7 @@ sub chargeback_activity_request {
     );
     ## set the authentication data
     tie my %authentication, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content  => \%content,
         user     => 'login',
         password => 'password',
@@ -1704,6 +1866,27 @@ sub chargeback_activity_request {
     return \@response_list;
 }
 
+=head2 chargeback_update_request
+
+Return a arrayref that contains a list of Business::OnlinePayment::Litle::ChargebackActivityResponse objects
+
+ $tx->content(
+  login                => 'testdrive',
+  password             => '123qwe',
+  case_id              => '1600010045',
+  merchant_activity_id => '1555',
+  activity             => 'Merchant Accepts Liability',
+ );
+
+ $tx->chargeback_update_request();
+
+ $tx->result_code(); # 0 - success, 1 invalid xml
+ $tx->error_message(); # Text version of the error message, if any
+ $tx->phoenixTxnId(); # Unique identifier provided by Litle.
+ $tx->is_success(); # Boolean, did the request work
+
+=cut
+
 sub chargeback_update_request {
     my ( $self ) = @_;
     my $post_data;
@@ -1729,7 +1912,7 @@ sub chargeback_update_request {
     );
     ## set the authentication data
     tie my %authentication, 'Tie::IxHash',
-      $self->revmap_fields(
+      $self->_revmap_fields(
         content  => \%content,
         user     => 'login',
         password => 'password',
@@ -1879,7 +2062,6 @@ by the Free Software Foundation; or the Artistic License.
 
 See http://dev.perl.org/licenses/ for more information.
 
-=back
 
 
 =head1 SEE ALSO
